package de.uni.leipzig.informatik.pacosy.datastructure.dao;

import de.uni.leipzig.informatik.pacosy.datastructure.api.INode;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITreeEdge;

public class Edge implements ITreeEdge {

	Node source;
	Node target;
	double length;
	double scaledLength;
	int bootstrap;

	public Edge() {
		this.source = null;
		this.target = null;
		this.length = 1d;
		this.bootstrap = 0;
	}

	@Override
	public Node getSource() {
		return this.source;
	}

	@Override
	public void setSource(INode source) {
		this.source = (Node) source;
		updateNodes();
		
	}

	@Override
	public Node getTarget() {
		return this.target;
	}

	@Override
	public void setTarget(INode target) {
		this.target = (Node) target;
		updateNodes();
	}
	
	private void updateNodes(){
		if (this.source!=null && this.target!=null){
			this.target.setParent(this.source);
			this.source.addChild(this.target);
		}
	}

	@Override
	public double getLength() {
		return this.length;
	}

	@Override
	public void setLength(double length) {
		this.length = length;
	}
	
	public double getScaledLength() {
		return scaledLength;
	}

	public void setScaledLength(double scaledLength) {
		this.scaledLength = scaledLength;
	}

	@Override
	public int getBootstrap() {
		return this.bootstrap;
	}

	@Override
	public void setBootstrap(int bootstrap) {
		this.bootstrap = bootstrap;
	}
	
	public void swap(){
		Node newTarget = this.source;
		Node newSource = this.target;
		
		this.source.getChildren().remove(this.target);
		
		newTarget.setParent(newSource);
		newSource.addChild(newTarget);
		
		this.source = newSource;
		this.target = newTarget;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + bootstrap;
		long temp;
		temp = Double.doubleToLongBits(length);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + ((source == null) ? 0 : source.hashCode());
		result = prime * result + ((target == null) ? 0 : target.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Edge other = (Edge) obj;
		if (bootstrap != other.bootstrap)
			return false;
		if (Double.doubleToLongBits(length) != Double
				.doubleToLongBits(other.length))
			return false;
		if (source == null) {
			if (other.source != null)
				return false;
		} else if (!source.equals(other.source))
			return false;
		if (target == null) {
			if (other.target != null)
				return false;
		} else if (!target.equals(other.target))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "(" + (this.source != null ? this.source.getId() : "") + ","
				+ (this.target != null ? this.target.getId() : "") + ")"
				+ (this.length!=0d?":"+this.length:"")
				+ (this.bootstrap!=0?"["+this.bootstrap+"]":"");
	}

}
