package de.uni.leipzig.informatik.pacosy.datastructure.dao;

import java.util.ArrayList;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.ITreeNode;

public class Node implements ITreeNode {

	String id;
	Node parent;
	List<Node> children;
	Map<Node,Double> associations;
	int number;
	int subtreeSize;

	public Node() {
		this.id = "";
		this.parent = null;
		this.children = new ArrayList<Node>();
		this.associations = new IdentityHashMap<Node, Double>();
		this.subtreeSize = -1;
	}

	@Override
	public String getId() {
		return this.id;
	}

	@Override
	public void setId(String id) {
		this.id = id;

	}

	public Node getParent() {
		return this.parent;
	}

	public void setParent(ITreeNode parent) {
		this.parent = (Node) parent;
	}

	public List<Node> getChildren() {
		return this.children;
	}

	public void setChildren(List<? extends ITreeNode> children) {
		this.children.clear();
		for (ITreeNode child : children){
			this.children.add((Node)child);
			
		}
	}

	public void addChild(ITreeNode child) {
		this.children.add((Node) child);

	}
	
	public Map<Node, Double> getAssociations() {
		return associations;
	}

	public void setAssociations(Map<Node, Double> associations) {
		this.associations = associations;
	}
	
	public void addAssociations(Node node, double strength) {
		this.associations.put(node, strength);
	}
	
	public boolean isLeaf(){
		return this.children.size()==0;
	}
	
	public int getNumber(){
		return this.number;
	}
	
	public int renumber(int number){
		this.number = number;
		for (Node child : this.children){
			number = child.renumber(number+1);
		}
		return number;
	}
	
	public int getSubtreeSize(){
		if (this.subtreeSize==-1)
		{
			computeSubtreeSize();
		}
		return this.subtreeSize;
	}
	
	public void computeSubtreeSize(){
		int size = 1;
		for (Node child : this.children){
			size += child.getSubtreeSize();
		}
		this.subtreeSize = size;
	}
	
	public List<Node> getSiblings(){
		List<Node> siblings = new ArrayList<Node>();
		if (this.getParent()!= null){
			siblings.addAll(this.parent.getChildren());
			siblings.remove(this);
		}
		return siblings;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + number;
		return result;
	}

//	@Override
//	public boolean equals(Object obj) {
//		if (this == obj)
//			return true;
//		if (obj == null)
//			return false;
//		if (getClass() != obj.getClass())
//			return false;
//		Node other = (Node) obj;
//		if (id == null) {
//			if (other.id != null)
//				return false;
//		} else if (!id.equals(other.id))
//			return false;
//		if (number != other.number)
//			return false;
//		return true;
//	}

	@Override
	public String toString() {
		return id;
		//return children.isEmpty() ? id : children + id;
	}

	public boolean isIncludedInSubtree(Node root) {
		Node parent = this;
		while (parent!=null){
			if (parent.equals(root)){
				return true;
			}
			parent = parent.getParent();
		}
		return false;
	}

}
