package de.uni.leipzig.informatik.pacosy.datastructure.dao;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import de.uni.leipzig.informatik.pacosy.datastructure.api.IAssociation;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITanglegramm;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;

public class Tanglegramm implements ITanglegramm {

	private Map<String, Tree> trees;
	private Set<Association> associations;
	private Tree defaultTree;
	
	public Tanglegramm(){
		this.trees = new HashMap<String, Tree>();
		this.defaultTree = null;
		this.associations = new HashSet<Association>();
	}
	
	@Override
	public Map<String, Tree> getTrees() {
		return this.trees;
	}
	
	@Override
	public void setTrees(Map<String, ? extends ITree> trees) {
		this.trees.clear();
		for (Map.Entry<String, ? extends ITree> entry : trees.entrySet()){
			this.trees.put(entry.getKey(), (Tree)entry.getValue());
		}
	}

	@Override
	public Tree getDefaultTree() {
		return this.defaultTree;
	}
	
	@Override
	public void setDefaultTree(ITree defaultTree) {
		this.defaultTree = (Tree) defaultTree;
		
	}
	
	@Override
	public Collection<Association> getAssociations() {
		return this.associations;
	}
	
	@Override
	public void setAssociations(Collection<? extends IAssociation> associations) {
		this.associations.clear();
		for (IAssociation association : associations){
			this.associations.add((Association)association);
		}
	}
	
	@Override
	public void update(){
		for (Association association : this.associations){
			association.update();
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((associations == null) ? 0 : associations.hashCode());
		result = prime * result
				+ ((defaultTree == null) ? 0 : defaultTree.hashCode());
		result = prime * result + ((trees == null) ? 0 : trees.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Tanglegramm other = (Tanglegramm) obj;
		if (associations == null) {
			if (other.associations != null)
				return false;
		} else if (!associations.equals(other.associations))
			return false;
		if (defaultTree == null) {
			if (other.defaultTree != null)
				return false;
		} else if (!defaultTree.equals(other.defaultTree))
			return false;
		if (trees == null) {
			if (other.trees != null)
				return false;
		} else if (!trees.equals(other.trees))
			return false;
		return true;
	}

}
