/**
 * 
 */
package de.uni.leipzig.informatik.pacosy.datastructure.util;

import de.uni.leipzig.informatik.pacosy.datastructure.dao.Edge;
import de.uni.leipzig.informatik.pacosy.datastructure.dao.Node;
import de.uni.leipzig.informatik.pacosy.datastructure.dao.Tree;


/**
 * @author Nicolas Wieseke
 * 
 */
public class NodeUtil {

	/**
	 * @param childCandidate
	 * @param parentCandidate
	 * @return true if the childCandidate is a child of the parentCandidate
	 */
	public static boolean isChild(Node childCandidate, Node parentCandidate) {
		return childCandidate.getNumber() > parentCandidate.getNumber()
				&& childCandidate.getNumber() < parentCandidate.getNumber()
						+ parentCandidate.getSubtreeSize();
	}

	/**
	 * @param node1
	 * @param node2
	 * @return the common ancestor of both nodes
	 */
	public static Node getCommonAncestor(Node node1, Node node2) {
		if (node1 == null || node2 == null)
			return null;
		if (node1.equals(node2))
			return node1;
		else if (node2.getParent() == null || isChild(node1, node2))
			return node2;
		else if (node1.getParent() == null || isChild(node2, node1))
			return node1;
		else
			return getCommonAncestor(node1.getParent(), node2.getParent());
	}

	/**
	 * @param node1
	 * @param node2
	 * @return the path length between node1 and node2
	 */
	public static int getPathLength(Node node1, Node node2) {
		Node lca = getCommonAncestor(node1, node2);
		if (lca == null) {
			return -1;
		}
		int counter = 0;
		Node parent = node1;
		while (parent != null && !parent.equals(lca)) {
			counter++;
			parent = parent.getParent();
		}
		parent = node2;
		while (parent != null && !parent.equals(lca)) {
			counter++;
			parent = parent.getParent();
		}
		return counter;
	}

	/**
	 * @param node
	 * @return the root node of the tree containing the node
	 */
	public static Node getRootNode(Node node) {
		while (node.getParent()!=null){
			node = node.getParent();
		}
		return node;
	}

	/**
	 * @param node
	 * @return
	 */
	public static String getEdgeList(Node node) {
		String edgeList = "";
		for (Node child : node.getChildren()) {
			edgeList += child.getNumber() + " " + node.getNumber() + "\n";
			edgeList += getEdgeList(child);
		}
		return edgeList;
	}
	
	/**
	 * @param node
	 * @return
	 */
	public static double getMaxDepth(Node node, Tree tree){
		double depth = 0d;
		for (Node child : node.getChildren()) {
			Edge edge = tree.getEdge(node.getId(), child.getId());
			depth = Math.max(depth, getMaxDepth(child, tree)+edge.getLength());
		}
		return depth;
	}
	
	/**
	 * @param node
	 * @return
	 */
	public static int getMaxOutdegree(Node node) {
		int maxOutdegree = node.getChildren().size();
		for (Node child : node.getChildren()){
			maxOutdegree = Math.max(maxOutdegree, getMaxOutdegree(child));
		}
		return maxOutdegree;
	}

	public static double getMinBranchLength(Tree tree) {
		double minBranchLength = 1;
		for (Edge edge : tree.getEdges()){
			double branchLength = edge.getLength();
			if (branchLength>0){
				minBranchLength = Math.min(minBranchLength, branchLength);
			}
		}
		return minBranchLength;
	}

	public static double getMaxBranchLength(Tree tree) {
		double maxBranchLength = Double.MIN_VALUE;
		for (Edge edge : tree.getEdges()){
			double branchLength = edge.getLength();
			if (branchLength>0){
				maxBranchLength = Math.max(maxBranchLength, branchLength);
			}
		}
		return maxBranchLength;
	}
}
