package de.uni.leipzig.informatik.pacosy.nexus;

import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;

import de.uni.leipzig.informatik.pacosy.datastructure.api.factory.IDaoFactory;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.block.parser.INexusBlockParser;
import de.uni.leipzig.informatik.pacosy.nexus.command.parser.INexusCommandParser;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusException;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;
import de.uni.leipzig.informatik.pacosy.nexus.parser.NexusStructureParser;

public class Nexus {

	private String filename;
	private Map<String, NexusBlock> blocks;

	public Nexus(String filename) {
		this.filename = filename;
		this.blocks = new HashMap<String, NexusBlock>();
	}

	public void read() throws NexusFormatException {
		String data = "";
		try {
			data = IOUtils.toString(new FileReader(filename));
		} catch (IOException e) {
			throw new NexusFormatException("Could not read the file "
					+ this.filename, e);
		}
		NexusStructureParser structureParser = new NexusStructureParser();
		List<NexusBlock> rawBlocks = structureParser.deserialize(data);

		Iterator<NexusBlock> iterator = rawBlocks.iterator();
		while (iterator.hasNext()) {
			NexusBlock rawBlock = iterator.next();
			String blockType = rawBlock.getBlockType();
			NexusBlock block = this.blocks.get(blockType);
			if (block == null) {
				this.blocks.put(blockType, rawBlock);
			} else {
				block.read(rawBlock, this.blocks);
			}
		}
	}

	public void addNexusBlock(String blockType, NexusBlock block) {
		this.blocks.put(blockType.toUpperCase(), block);
	}

	public void removeNexusBlock(String blockType) {
		this.blocks.remove(blockType.toUpperCase());
	}

	public void setNexusBlockParser(String blockType,
			INexusBlockParser blockParser) {
		if (this.blocks.containsKey(blockType.toUpperCase())) {
			this.blocks.get(blockType.toUpperCase())
					.setBlockParser(blockParser);
		}
	}

	public void setNexusBlockDaoFactory(String blockType, IDaoFactory daoFactory) {
		if (this.blocks.containsKey(blockType.toUpperCase())) {
			this.blocks.get(blockType.toUpperCase()).setDaoFactory(daoFactory);
		}
	}

	public void setNexusCommandParser(String blockType, String commandName,
			INexusCommandParser commandParser) throws NexusException {
		if (this.blocks.containsKey(blockType.toUpperCase())) {
			this.blocks.get(blockType.toUpperCase()).addNexusCommandParser(
					commandName.toUpperCase(), commandParser);
		}
	}

	public String getFilename() {
		return filename;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public Map<String, NexusBlock> getBlocks() {
		return blocks;
	}

	public void setBlocks(Map<String, NexusBlock> blocks) {
		this.blocks = blocks;
	}

}
