package de.uni.leipzig.informatik.pacosy.nexus.block;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.factory.IDaoFactory;
import de.uni.leipzig.informatik.pacosy.nexus.Token;
import de.uni.leipzig.informatik.pacosy.nexus.block.parser.DefaultNexusBlockParser;
import de.uni.leipzig.informatik.pacosy.nexus.block.parser.INexusBlockParser;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.parser.INexusCommandParser;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusException;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusBlock {

	private String blockType;
	private String blockName;
	private List<Token> tokens;
	private List<NexusCommand> commands;
	private INexusBlockParser blockParser;
	private IDaoFactory daoFactory;
	
	public NexusBlock(){
		this.blockType = "";
		this.blockName = "";
		this.tokens = new ArrayList<Token>();
		this.commands = new ArrayList<NexusCommand>();
		initialize();
	}
	
	protected void initialize(){
		this.blockParser = DefaultNexusBlockParser.getSingleton();
	}
	
	public void read(NexusBlock rawBlock, Map<String, NexusBlock> blocks)
			throws NexusFormatException {
		if (rawBlock == null){
			throw new NexusFormatException("specified block missing");
		}
		this.blockType = rawBlock.blockType;
		this.blockName = rawBlock.blockName;
		this.tokens = rawBlock.tokens;
		this.commands = getBlockParser().deserializeCommands(rawBlock, blocks);
	}
	
	public Object getCommandData(String commandName){
		for (NexusCommand command : this.commands){
			if (command.getCommandName().equalsIgnoreCase(commandName)){
				return command.getData();
			}
		}
		return null;
	}
	
	public INexusBlockParser getBlockParser() {
		return this.blockParser;
	}

	public void setBlockParser(INexusBlockParser blockParser) {
		this.blockParser = blockParser;
	}
	
	public IDaoFactory getDaoFactory() {
		return this.daoFactory;
	}

	public void setDaoFactory(IDaoFactory daoFactory) {
		this.daoFactory = daoFactory;
	}

	public void addNexusCommandParser(String commandName, INexusCommandParser commandParser) {
		this.blockParser.registerCommandParser(commandName.toUpperCase(), commandParser);
	}
	
	public void removeNexusCommandParser(String commandName) throws NexusException{
		this.blockParser.unregisterCommandParser(commandName.toUpperCase());
	}
	
	public String getBlockType() {
		return this.blockType.toUpperCase();
	}

	public void setBlockType(String blockType) {
		this.blockType = blockType;
	}
	
	public String getBlockName() {
		return this.blockName;
	}

	public void setBlockName(String blockName) {
		this.blockName = blockName;
	}
	
	public List<Token> getTokens() {
		return this.tokens;
	}

	public void setTokens(List<Token> tokens) {
		this.tokens = tokens;
	}

	public List<NexusCommand> getCommands() {
		return commands;
	}

	public void setCommands(List<NexusCommand> commands) {
		this.commands = commands;
	}
	
	public void addCommand(NexusCommand command) {
		this.commands.add(command);
	}

	@Override
	public String toString() {
		return blockType + ": " + this.commands;
	}

	
}
