package de.uni.leipzig.informatik.pacosy.nexus.block;

import java.util.HashMap;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;
import de.uni.leipzig.informatik.pacosy.nexus.block.parser.NexusTreesBlockParser;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusMapOfStringsStyleCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusTreeCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusTreesBlock extends NexusBlock {

	private Map<String, String> translate;
	private Map<String, ITree> trees;
	private ITree defaultTree;
	
	protected void initialize(){
		this.translate = new HashMap<String, String>();
		this.trees = new HashMap<String, ITree>();
		setBlockParser(NexusTreesBlockParser.getSingleton());
	}
	
	@Override
	public void read(NexusBlock rawBlock, Map<String, NexusBlock> blocks)
			throws NexusFormatException {
		super.read(rawBlock, blocks);
		for (NexusCommand command : getCommands()) {
			if (command.getCommandName().equalsIgnoreCase("TRANSLATE") && command instanceof NexusMapOfStringsStyleCommand){
				this.translate = ((NexusMapOfStringsStyleCommand)command).getMap();
			} else if (command.getCommandName().equalsIgnoreCase("TREE") && command instanceof NexusTreeCommand){
				NexusTreeCommand treeCommand = (NexusTreeCommand)command;
				this.trees.put(treeCommand.getName(), treeCommand.getTree());
				if (treeCommand.isDefaultTree()){
					this.defaultTree = treeCommand.getTree();
				}
			}
		}
	}

	public Map<String, String> getTranslate() {
		return translate;
	}

	public Map<String, ITree> getTrees() {
		return trees;
	}

	public ITree getDefaultTree() {
		return defaultTree;
	}
	
}
