package de.uni.leipzig.informatik.pacosy.nexus.block.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.factory.IDaoFactory;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.parser.DefaultNexusCommandParser;
import de.uni.leipzig.informatik.pacosy.nexus.command.parser.INexusCommandParser;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class DefaultNexusBlockParser implements INexusBlockParser {

	private static DefaultNexusBlockParser singleton;
	
	public static DefaultNexusBlockParser getSingleton(){
		if (singleton == null){
			singleton = new DefaultNexusBlockParser();
		}
		return singleton;
	}
	
	private IDaoFactory daoFactory;
	private Map<String, INexusCommandParser> commandParsers;

	protected DefaultNexusBlockParser() {
		this.commandParsers = new HashMap<String, INexusCommandParser>();
	}

	@Override
	public void registerCommandParser(String id,
			INexusCommandParser commandParser) {
		if (commandParser.getDaoFactory() == null) {
			commandParser.setDaoFactory(this.daoFactory);
		}
		this.commandParsers.put(id.toUpperCase(), commandParser);

	}

	@Override
	public void unregisterCommandParser(String id) {
		this.commandParsers.remove(id.toUpperCase());
	}

	@Override
	public INexusCommandParser getNexusCommandParser(String id) {
		if (this.commandParsers.containsKey(id.toUpperCase())) {
			return this.commandParsers.get(id.toUpperCase());
		} else {
			return DefaultNexusCommandParser.getSingleton();
		}
	}

	@Override
	public IDaoFactory getDaoFactory() {
		return this.daoFactory;
	}

	@Override
	public void setDaoFactory(IDaoFactory daoFactory) {
		this.daoFactory = daoFactory;
		for (INexusCommandParser commandParser : this.commandParsers.values()) {
			commandParser.setDaoFactory(daoFactory);
		}
	}
	
	@Override
	public List<NexusCommand> deserializeCommands(NexusBlock rawBlock,
			Map<String, NexusBlock> blocks) throws NexusFormatException {
		List<NexusCommand> commands = new ArrayList<NexusCommand>();
		for (NexusCommand rawCommand : rawBlock.getCommands()) {
			String commandName = rawCommand.getCommandName();
			commands.add(getNexusCommandParser(commandName).deserialize(
					rawCommand, commands, blocks));
		}
		return commands;
	}

}
