package de.uni.leipzig.informatik.pacosy.nexus.command.parser;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.IAssociation;
import de.uni.leipzig.informatik.pacosy.datastructure.api.INode;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;
import de.uni.leipzig.informatik.pacosy.datastructure.api.factory.ICophylogenyFactory;
import de.uni.leipzig.informatik.pacosy.nexus.Token;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusTreesBlock;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCophylogenyAssociationsCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusCophylogenyAssociationMatrixCommandParser extends
		DefaultNexusCommandParser {

	protected static NexusCophylogenyAssociationMatrixCommandParser singleton;
	
	public static NexusCophylogenyAssociationMatrixCommandParser getSingleton(){
		if (singleton == null){
			singleton = new NexusCophylogenyAssociationMatrixCommandParser();
		}
		return singleton;
	}
	
	private NexusCophylogenyAssociationMatrixCommandParser(){}
	
	@Override
	public ICophylogenyFactory getDaoFactory() {
		return (ICophylogenyFactory)super.getDaoFactory();
	}
	
	@Override
	public NexusCommand deserialize(NexusCommand rawCommand, List<NexusCommand> commands, Map<String, NexusBlock> blocks) throws NexusFormatException {
		Map<String, ITree> trees = ((NexusTreesBlock)blocks.get("TREES")).getTrees();
		List<IAssociation> list = new ArrayList<IAssociation>();
		Iterator<Token> iterator = rawCommand.getTokens().iterator();
		List<INode> species1List = new ArrayList<INode>();
		INode species2 = null;
		Token token = null;
		boolean species1ListDone = false;
		boolean species2Done = false;
		int species1Counter = 0;
		while (iterator.hasNext()) {
			token = iterator.next();
			if (!token.isComment()) {
				if (!token.getValue().equals(",")) {
					if (!species1ListDone) {
						try {
							species1List.add(getSpecies(token.getValue(), trees));
						} catch (Exception e) {
							throw new NexusFormatException(
									"unknown species identifier " + token.getValue() + " at " + token.getPosition());
						}
					} else if (!species2Done) {
						try {
							species2 = getSpecies(token.getValue(), trees);
							species2Done = true;
							species1Counter = 0;
						} catch (Exception e) {
							throw new NexusFormatException(
									"unknown species identifier " + token.getValue() + " at " + token.getPosition());
						}
					} else {
						try {
							double strength = Double.parseDouble(token.getValue());
							INode species1 = species1List.get(species1Counter);
							IAssociation association = getDaoFactory().createAssociation();
							association.setPrimaryNode(species1);
							association.setSecondaryNode(species2);
							association.setStrength(strength);
							list.add(association);
							species1Counter++;
						} catch (NumberFormatException e) {
							throw new NexusFormatException(
									"association strength expected at " + token.getPosition(),e);
						} catch (IndexOutOfBoundsException e) {
							throw new NexusFormatException(
									"',' sign expected at " + token.getPosition());
						}
					}
				} else {
					species1ListDone=true;
					species2Done=false;
				}
			}
		}
		return new NexusCophylogenyAssociationsCommand(rawCommand, list);
	}
	
	private INode getSpecies(String speciesId, Map<String, ITree> trees) throws NexusFormatException{
		for (ITree tree : trees.values()){
			INode species = tree.getNode(speciesId);
			if (species != null){
				return species;
			}
		}
		throw new NexusFormatException(	);
	}

}
