package de.uni.leipzig.informatik.pacosy.nexus.command.parser;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.IAssociation;
import de.uni.leipzig.informatik.pacosy.datastructure.api.INode;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;
import de.uni.leipzig.informatik.pacosy.datastructure.api.factory.ICophylogenyFactory;
import de.uni.leipzig.informatik.pacosy.nexus.Token;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusTreesBlock;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCophylogenyAssociationsCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusCophylogenyAssociationsCommandParser extends
		DefaultNexusCommandParser {

	protected static NexusCophylogenyAssociationsCommandParser singleton;
	
	public static NexusCophylogenyAssociationsCommandParser getSingleton(){
		if (singleton == null){
			singleton = new NexusCophylogenyAssociationsCommandParser();
		}
		return singleton;
	}
	
	private NexusCophylogenyAssociationsCommandParser(){}
	
	@Override
	public ICophylogenyFactory getDaoFactory() {
		return (ICophylogenyFactory)super.getDaoFactory();
	}
	
	@Override
	public NexusCommand deserialize(NexusCommand rawCommand, List<NexusCommand> commands, Map<String, NexusBlock> blocks) throws NexusFormatException {
		Map<String, ITree> trees = ((NexusTreesBlock)blocks.get("TREES")).getTrees();
		List<IAssociation> list = new ArrayList<IAssociation>();
		Iterator<Token> iterator = rawCommand.getTokens().iterator();
		INode species1 = null;
		INode species2 = null;
		Double strength = null;
		Token token = null;
		while (iterator.hasNext()) {
			token = iterator.next();
			if (!token.isComment()) {
				if (!token.getValue().equals(",")) {
					if (species1==null) {
						try {
							species1 = getSpecies(token.getValue(), trees);
						} catch (Exception e) {
							throw new NexusFormatException(
									"unknown species identifier " + token.getValue() + " at " + token.getPosition());
						}
					} else if (species2==null) {
						try {
							species2 = getSpecies(token.getValue(), trees);
						} catch (Exception e) {
							throw new NexusFormatException(
									"unknown species identifier " + token.getValue() + " at " + token.getPosition());
						}
					} else if (strength==null) {
						try {
							strength = Double.parseDouble(token.getValue());
						} catch (NumberFormatException e) {
							throw new NexusFormatException(
									"association strength expected at " + token.getPosition(),e);
						}
					} else {
						throw new NexusFormatException(
								"',' sign expected at " + token.getPosition());
					}
				} else if (species1!=null) {
					if (species2!=null) {
						if (strength!=null){
							IAssociation association = getDaoFactory().createAssociation();
							association.setPrimaryNode(species1);
							association.setSecondaryNode(species2);
							association.setStrength(strength);
							list.add(association);
							species1 = null;
							species2 = null;
							strength = null;
						} else {
							throw new NexusFormatException(
									"association strength expected at " + token.getPosition());
						}
					} else {
						throw new NexusFormatException(
								"2nd species identifier expected at " + token.getPosition());
					}
				} else {
					throw new NexusFormatException(
							"species identifier expected at " + token.getPosition());
				}
			}
		}
		if (species1!=null) {
			if (species2!=null) {
				if (strength!=null){
					IAssociation association = getDaoFactory().createAssociation();
					association.setPrimaryNode(species1);
					association.setSecondaryNode(species2);
					association.setStrength(strength);
					list.add(association);
				} else {
					throw new NexusFormatException(
							"association strength expected at " + token.getPosition());
				}
			} else {
				throw new NexusFormatException(
						"2nd species identifier expected at " + token.getPosition());
			}
		}
		return new NexusCophylogenyAssociationsCommand(rawCommand, list);
	}
	
	private INode getSpecies(String speciesId, Map<String, ITree> trees) throws NexusFormatException{
		for (ITree tree : trees.values()){
			INode species = tree.getNode(speciesId);
			if (species != null){
				return species;
			}
		}
		throw new NexusFormatException(	);
	}

}
